@@ -1,9 +1,17 @@
 /*******************************************************************
- * Author: Mathias Hofmann
- *         Michael Hoehle <hoehle@stat.uni-muenchen.de>
- *         Volker Schmid
- *         Daniel Sabanes Bove <daniel.sabanesbove@ifspm.uzh.ch>
+ * Authors:
+ *   Mathias Hofmann
+ *   Michael Hoehle <hoehle@stat.uni-muenchen.de>
+ *   Volker Schmid
+ * Contributors:
+ *   Michaela Paul
+ *   Daniel Sabanes Bove
+ *   Sebastian Meyer
  * History:
+ *   October 2022 (SM) -- dropped deprecated sprintf and a lot of unused code
+ *                        and fixed a memory leak
+ *   July 2016 (SM) -- dropped deprecated "register" storage class specifier
+ *   April 2012 (SM) -- replaced exit() calls by Rf_error()
  *   March 2012 (DSB) -- changed long types to int to be in accordance with R
  *                       (we observed bad allocations in 64 bit machines)
  *   May 2010 (DSB) -- modified from Oct 2008
@@ -11,8 +19,7 @@
  * Markov Chain Monte Carlo (MCMC) estimation in the Branching Process
  * like Epidemic Model. Instead of a slow R solution this code
  * provides a faster C++ solution. Can be invoked through R or be
- * programmed as a librrary. This code uses the Gnu Scientific Library
- * (GSL) available from http://sources.redhat.com/gsl/
+ * programmed as a librrary.
  *
  * For now this code is quick & dirty. A more OO framework would be nice
  * to enable better programming, but this will probably be speedwise slower.
@@ -212,44 +219,7 @@ double epsilon_b=0.001;
 
 
 /*********************************************************************
- * Compute sum from 1 to I and 1 to n of a vektor with indices 0,...,I 
- * of a vektor with indices 0,...,n
- * Parameters:
- *
- * X a vector with indices 0,..,I of a vector with indices 0,...,n
- * I "length" of vector (true length due to zero indice is I+1)
- *********************************************************************/
-double sumIn(const LongMatrix& X, int I, int n) {
-  double res = 0;
-  for (register int i=1; i<=I; i++){
-    for (register int t=1; t<=n; t++) {
-      res += X[i][t]; 
-    }
-  }
-  return(res);
-}
-
-/*********************************************************************
- * Compute sum from 1 to I and 1 to n of a vektor with indices 0,...,I 
- * of a vektor with indices 0,...,n
- * This is the double version
- * Parameters:
- *
- * X a vector with indices 0,..,I of a vector with indices 0,...,n
- * I "length" of vector (true length due to zero indice is I+1)
- *********************************************************************/
-double sumIn(const DoubleMatrix& X, int I, int n) {
-  double res = 0;
-  for (register int i=1; i<=I; i++){
-    for (register int t=1; t<=n; t++) {
-      res += X[i][t]; 
-    }
-  }
-  return(res);
-}
-
-/*********************************************************************
- * Compute sum from 1 to I and 1 to n of a vektor with indices 0,...,I 
+ * Compute sum from 1 to I and 2 to n of a vektor with indices 0,...,I 
  * of a vektor with indices 0,...,n
  * Parameters:
  *
@@ -258,8 +228,8 @@ double sumIn(const DoubleMatrix& X, int I, int n) {
  *********************************************************************/
 double sumIn2(const LongMatrix& X, int I, int n) {
   double res = 0;
-  for (register int i=1; i<=I; i++){
-    for (register int t=2; t<=n; t++) {
+  for (int i=1; i<=I; i++){
+    for (int t=2; t<=n; t++) {
       res += X[i][t]; 
     }
   }
@@ -268,7 +238,7 @@ double sumIn2(const LongMatrix& X, int I, int n) {
 
 
 /*********************************************************************
- * Compute sum from 1 to I and 1 to n of a vektor with indices 0,...,I 
+ * Compute sum from 1 to I and 2 to n of a vektor with indices 0,...,I 
  * of a vektor with indices 0,...,n
  * This is the double version
  * Parameters:
@@ -278,8 +248,8 @@ double sumIn2(const LongMatrix& X, int I, int n) {
  *********************************************************************/
 double sumIn2(const DoubleMatrix& X, int I, int n) {
   double res = 0;
-  for (register int i=1; i<=I; i++){
-    for (register int t=2; t<=n; t++) {
+  for (int i=1; i<=I; i++){
+    for (int t=2; t<=n; t++) {
       res += X[i][t]; 
     }
   }
@@ -287,52 +257,6 @@ double sumIn2(const DoubleMatrix& X, int I, int n) {
 }
 
 
-/*********************************************************************
- * Compute sum from 1 to I of a vektor with indices 0,...,I 
- * of a vektor with indices 0,...,n
- * Parameters:
- *
- * X a vector with indices 0,..,I of a vector with indices 0,...,n
- * I "length" of vector (true length due to zero indice is I+1)
- *********************************************************************/
-double sumI1(const LongMatrix& X, int I, int t) {
-  double res = 0;
-  for (register int i=1; i<=I; i++) { res += X[i][t]; }
-  return(res);
-}
-
-/*********************************************************************
- * Compute sum from 1 to I of a vektor with indices 0,...,I 
- * of a vektor with indices 0,...,n
- * This is the double version
- * Parameters:
- *
- * X a vector with indices 0,..,I of a vector with indices 0,...,n
- * I "length" of vector (true length due to zero indice is I+1)
- *********************************************************************/
-double sumI1(const DoubleMatrix& X, int I, int t) {
-  double res = 0;
-  for (register int i=1; i<=I; i++) { res += X[i][t]; }
-  return(res);
-}
-
-/*********************************************************************
- * factorial function
- *********************************************************************/
-long factorial(long x){
-  long fac=1;
-  if(x<0){ Rf_error("negative value passed to factorial function\n");}
-  else{
-    if(x==0){fac=1;}
-    else{
-      for(int i=1;i<=x;i++){
-        fac*=i;
-      }
-    }
-  }
-  return(fac);
-}
-
 /*********************************************************************
  * logit function
  *********************************************************************/
@@ -346,8 +270,6 @@ double logit(double y){
 }
 
 
-
-
 /*********************************************************************
  * inverse logit function
  *********************************************************************/
@@ -368,31 +290,6 @@ double invlogitd(double y){
 }
 
 
-
-
-/*********************************************************************
- * Makes one Metropolis-Hastings update step, log-scale
- *********************************************************************/
-double updateMHlog(double &par, double parStar, double logFpar, double logFparStar, double &acceptedpar) {
-  double accpar = exp(logFparStar - logFpar); 
-  if (gsl_rng_uniform() <= accpar) {par = parStar; acceptedpar++;}
-  return(0);
-}
-
-/*********************************************************************
- * Makes one Metropolis-Hastings update step
- *********************************************************************/
-double updateMH(double &par, double parStar, double Fpar, double FparStar, double &acceptedpar) {
-  double accpar = FparStar/Fpar; 
-  if (gsl_rng_uniform() <= accpar) {par = parStar; acceptedpar++;}
-  return(0);
-}
-
-
-
-
-
-
 /*********************************************************************
  * Tunes a parameter
  *********************************************************************/
@@ -408,203 +305,6 @@ double tune(double& parameter, double accepted, double samples, double& tunepar,
       return(0);
 }
 
-double ABS(double x)
-{
-  if (x>0){return x;}else{return -x;}
-}
-double MIN(double a, double b)
-{
-  if (a<b){return a;}else{return b;}
-}
-
-double xMx(double* Q, double* x, int noa, int b)
-{
-  double erg=0.0;
-  
-  for (int i=0; i<noa; i++)
-    {
-      for (int j=0; j<noa; j++)
-	{
-	  if (ABS(i-j) < b)
-	    {
-	      erg += x[i]*x[j]*Q[(int)(MIN(i,j)*b+ABS(i-j))];
-	      if (i==j)
-		{
-		  erg -= 0.5*x[i]*x[j]*Q[(int)(MIN(i,j)*b+ABS(i-j))];
-		}
-	    }
-	}
-    }
-
-  return erg;
-}
-
-double xMx2(double* Q, double* x, int n, int b)
-{
-  double erg=0.0;
-  
-  for (int i=0; i<=n; i++)
-    {
-      for (int j=0; j<=n; j++)
-	{
-	  if (ABS(i-j) < b)
-	    {
-	      erg += x[i]*x[j]*Q[(int)(MIN(i,j)*b+ABS(i-j))];
-	    }
-	}
-    }
-
-  return erg;
-}
-
-
-/* BERECHNET A-1, k ist matrixlaenge*/
-void invers(double* A,int k)
-{
-  DoubleVector ergebnis(k * k);
-  
-  if (k==1)
-    {
-      ergebnis[0] = 1.0/A[0];
-    }
-
-  if (k==2)
-    {
-      double det = A[0]*A[3]-A[1]*A[2];
-      ergebnis[0] = A[3]/det;
-      ergebnis[1] = 0.0-A[1]/det;
-      ergebnis[2] = 0.0-A[2]/det;
-      ergebnis[3] = A[0]/det;
-    }
-
-  if (k>2)
-    {
-      REprintf("Error in the twins.cc function invers()\n");
-    }
-
-  for (int i=0; i< k*k; i++)
-    {
-      A[i]=ergebnis[i];
-    }
-  
-  return;
-}
-
-void mxschreibe(double* A, int a, int b)
-{
-  for (int i=0; i<a; i++)
-    {
-      for (int j=0; j<b; j++)
-	{
-	  Rprintf("%f ", A[i*b+j]);
-	}
-      Rprintf("\n");
-    }
-  Rprintf("\n");
-  return;
-}
-
-
-int mxcheck(int n, const IntMatrix& matrix)
-{
-  int zs=0;
-  for (int i=0; i<n; i++)
-    {
-      zs=0;
-      for (int j=0; j<n; j++)
-	{
-	  zs+=matrix[i][j];
-	  if (matrix[i][j]!=matrix[j][i])
-	    {
-	      REprintf("Error: Matrix is not symmetric! (Row: %d, Column %d\n",i,j); 
-	      return 1;
-	    }
-	}
-      if (zs != 0)
-	{
-	  REprintf("Error: Row sum not zero in row %d",i,"\n");
-	  return 1;
-	}
-    }
-  return 0;
-}
-
-
-
-
-
-/*updatealphabeta
-  Erzeugt Normalverteilten Zufallsvektor der Laenge noa*/
-void gausssample(double* temp, int noa)
-{
-  
-  for (int i=0; i< noa; i++)
-    {
-      temp[i]=gsl_ran_gaussian(1);
-    }
-  
-  return;
-}
-
-double hyper(int rw, double* theta, double k_a, double k_b, int n)
-{
-
-  double aa;
-  double bb;
-  double kappa_neu=0;
-  
-  if (rw==1)
-    {
-      double summe = 0.0;
-      
-      aa = k_a + 0.5 * double(n-1-rw);
-      
-      for (int i=3; i <= n; i++)
-	{
-	  summe = summe + (theta[i] - theta[i-1]) * (theta[i] - theta[i-1]);
-	}
-      bb = k_b + 0.5 * summe;
-      
-      kappa_neu = gsl_ran_gamma(aa,1/bb);
-    }
-  if (rw==2)
-    {
-      
-      double dopp_diff;
-      double summe = 0.0;
-      
-      aa = k_a + 0.5 * double(n-1-rw);
-      
-      for (int i=3; i < n; i++)
-	{
-	  dopp_diff = theta[i-1] - 2*theta[i] + theta[i+1];
-	  summe = summe + dopp_diff * dopp_diff;
-	}
-      bb = k_b + 0.5 * summe;
-      
-      
-      
-      kappa_neu = gsl_ran_gamma( aa, 1/bb);
-    }
-  if (rw==0)
-    {
-      double summe = 0.0;
-      
-      aa = k_a + 0.5 * double(n-1-rw);
-      
-      for (int i=2; i <= n; i++){
-	summe = summe + (theta[i] * theta[i]);
-      }
-      
-      bb = k_b + 0.5 * summe;
-      
-      kappa_neu = gsl_ran_gamma(aa,1/bb);
-    }
-  
-  return kappa_neu;
-}
-
-
 double update_tau_alpha(const DoubleVector& alpha, int I, double aa, double bb, DoubleVector& xreg)
 {
   
@@ -618,73 +318,6 @@ double update_tau_alpha(const DoubleVector& alpha, int I, double aa, double bb,
 }
 
 
-double update_tau_gamma(const DoubleVector& alpha, int ncov, double aa, double bb)
-{
-  
-  aa += double(ncov);
-  for (int i=0; i<ncov; i++)
-    {
-      bb += alpha[i]*alpha[i];
-    }
-  double neu=gsl_ran_gamma( aa, 1/bb);
-  return neu;
-}
-
-void berechneQ(double* temp, int age_block, double kappa, int noa,int nop, double delta)
-{
-
-  if (age_block==1)
-    {
-      int index=0;
-      temp[index]=kappa+delta*nop;
-      index++;
-      temp[index]=-kappa;
-      index++;
-      for (int i=1; i<noa-1; i++)
-	{
-	  temp[index]=2*kappa+delta*nop;
-	  index++;
-	  temp[index]=-kappa;
-	  index++;
-	}
-      temp[index]=kappa+delta*nop;
-    }
-  if (age_block==2)
-    {
-      int index=0;
-      temp[index]=kappa+delta*nop;
-      index++;
-      temp[index]=-2*kappa;
-      index++;
-      temp[index]=kappa;
-      index++;
-      temp[index]=5*kappa+delta*nop;
-      index++;
-      temp[index]=-4*kappa;
-      index++;
-      temp[index]=kappa;
-      index++;
-      for (int i=2; i<noa-2; i++)
-	{
-	  temp[index]=6*kappa+delta*nop;
-	  index++;
-	  temp[index]=-4*kappa;
-	  index++;
-	  temp[index]=kappa;
-	  index++;
-	}
-      temp[index]=5*kappa+delta*nop;
-      index++;
-      temp[index]=-2*kappa;
-      index++;
-      index++;
-      temp[index]=kappa+delta*nop;
-    }
-
-  return;
-}
-
-
 
 double sumg(int ncov, const DoubleMatrix& xcov, DoubleVector& gamma, int t, int scov)
 {
@@ -697,8 +330,6 @@ double sumg(int ncov, const DoubleMatrix& xcov, DoubleVector& gamma, int t, int
 }
 
 
-
-
 void alphaupdate(DoubleVector& gamma, DoubleVector& alpha, DoubleVector& beta, DoubleVector& delta, const DoubleMatrix& lambda, double p, int I, int n, const LongMatrix& Y, const LongMatrix& X, long& acc_alpha, double taualpha, int ncov, const DoubleMatrix& xcov, DoubleVector& xreg, const DoubleMatrix& omega, const DoubleMatrix& omegaX, int scov, int mode){
 
   for (int i=1; i<=I; i++)
@@ -747,100 +378,6 @@ void alphaupdate(DoubleVector& gamma, DoubleVector& alpha, DoubleVector& beta, D
 
 
 
-void erzeuge_b_Q(DoubleVector& gamma  , double* my, double* Q, const DoubleVector& alpha, DoubleVector& delta, DoubleVector& beta, const LongMatrix& X, const LongMatrix& Z, const LongMatrix& Y,
-		 int n, int I, double taubeta, int rw, const DoubleMatrix& lambda, double p, const DoubleMatrix& xcov, int ncov, const DoubleMatrix& omega, const DoubleMatrix& omegaX,int scov, int mode)
-{
-  if (mode==1)
-    {
-      /* b-vektor des Proposals*/
-      for (int t=0;t<n; t++)
-	{
-	  my[t]=0.0;
-	  for (int i=1; i<=I; i++)
-	    {
-	      my[t]+=X[i][t+2];
-	      my[t]-=(1-beta[t+2])*omegaX[i][t+2]*delta[t+2]*(exp(sumg(ncov,xcov,gamma,t+2,scov)+alpha[i]+beta[t+2]));
-	    }
-	}
-    }
-  if (mode==2)
-    {
-      /* b-vektor des Proposals*/
-      for (int t=2;t<=n; t++)
-	{
-	  my[t-2]=0.0;
-	  for (int i=1; i<=I; i++)
-	    {
-	      my[t-2]+=Y[i][t];
-	      my[t-2]-=(1-beta[t])*(omega[i][t]*Z[i][t-1]*exp(sumg(ncov,xcov,gamma,t,scov)+alpha[i]+beta[t]));
-	    }
-	}
-      
-      
-    }
-  
-  /* Praezisionsmatrix*/
-  berechneQ(Q, rw, taubeta, n, 1, 0.0);
-  
-  if (mode==1)
-    {
-      for (int i=1; i<=I; i++)
-	{
-	  for (int t=0; t<n; t++)
-	    {
-	      Q[(rw+1)*(t)]+= omegaX[i][t+2]*delta[t+2]*exp(sumg(ncov,xcov,gamma,t+2,scov)+alpha[i]+beta[t+2]);
-	    }	
-	}
-    }
-  
-  if (mode==2)
-    {
-      for (int i=1; i<=I; i++)
-	{
-	  for (int t=2; t<=n; t++)
-	    {
-	      Q[(rw+1)*(t-2)]+= omega[i][t]*Z[i][t-1]*exp(sumg(ncov,xcov,gamma,t,scov)+alpha[i]+beta[t]);
-	    }
-	}	
-    }
-  
-  return;
-}
-
-
-void erzeuge_b_Q_2(double* my, double* Q, const DoubleVector& alpha, DoubleVector& beta, DoubleVector& gamma, DoubleVector& delta, const LongMatrix& X,
-		   int n, int I, double taubeta, int rw, const DoubleMatrix& xcov, int ncov, int scov, const DoubleMatrix& omega)
-{
-
-  /* b-vektor des Proposals*/
-  for (int t=0;t<=n; t++)
-    {
-      my[t]=0.0;
-      for (int i=1; i<=I; i++)
-	{
-	  my[t]+=X[i][t+2];
-	  my[t]-=(1-beta[t])*omega[i][t+2]*delta[t+2]*exp(sumg(ncov,xcov,gamma,t+2,scov)+alpha[i]+beta[t]);
-	}
-    }
-
-
-  /* Praezisionsmatrix*/
-  berechneQ(Q, rw, taubeta, n+1, 1, 0.0);
-
-  for (int i=1; i<=I; i++)
-    {
-      for (int t=0; t<=n; t++)
-	{
-	  Q[(rw+1)*(t)]+= omega[i][t+2]*delta[t+2]*exp(sumg(ncov,xcov,gamma,t+2,scov)+alpha[i]+beta[t]);
-	}	
-    }
-
-
-
-  return;
-}
-
-
 
 void machnu(DoubleVector& mu, const DoubleVector& alpha, DoubleVector& beta, DoubleVector& delta, DoubleMatrix& nu, int I, int n, int ncov, const DoubleMatrix& xcov, int scov)
 {
@@ -923,79 +460,6 @@ void update_gamma_j(int j, const DoubleVector& alpha, DoubleVector& beta, Double
 
 
 
-void update_beta_t(int t, const DoubleVector& alpha, DoubleVector& beta, DoubleVector& gamma, DoubleVector& delta, int ncov, const DoubleMatrix& xcov, const LongMatrix& X, int n, int I, double taubeta, long& acc_beta, const DoubleMatrix& omega, int scov)
-{
-  double h = 0;
-  double c = 0;
-  double d = 0;
-  for(int i=1;i<=I;i++){
-    h -= omega[i][t]*delta[t]*exp(alpha[i] + beta[t] + sumg(ncov,xcov,gamma,t,scov)); /* h ist h(beta[t]^0), beta ist \beta^0, betatStar ist \beta*/
-    c += X[i][t];
-  }
-  if(t==2){
-    c -= taubeta*(beta[t+2]-2*beta[t+1]);
-    d = taubeta;
-  }
-  if(t==3){
-    c -= taubeta*((beta[t+2]-2*beta[t+1]) + (-2*beta[t+1] - 2*beta[t-1]));
-    d = 5*taubeta;
-  }
-  if((t>=4)&&(t<=(n-2))){
-    c -= taubeta*((beta[t+2]-2*beta[t+1]) + (-2*beta[t+1] - 2*beta[t-1]) + (beta[t-2] - 2*beta[t-1]));
-    d = 6*taubeta;
-  }
-  if(t==(n-1)){
-    c -= taubeta*((-2*beta[t+1] - 2*beta[t-1]) + (beta[t-2] - 2*beta[t-1]));
-    d = 5*taubeta;
-  }
-  if(t==n){
-    c -= taubeta*(beta[t-2] - 2*beta[t-1]);
-    d = taubeta;
-  }
-
-
-
-  double s = sqrt(1/(d - h)); /* s ist s*/
-  double b = c + (1 - beta[t])*h;
-  double m = b*s*s;
-
-  double betatStar = gsl_ran_gaussian(s) + m;
-
-  double h2 = 0;
-  for(int i=1;i<=I;i++){
-    h2 -= omega[i][t]*delta[t]*exp(alpha[i] + betatStar + sumg(ncov,xcov,gamma,t,scov)); /* h2 ist h(beta[t])*/
-  }
-
-  double s2 = sqrt(1/(d - h2)); /* s2 ist s^0*/
-  double b2 = c + (1 - betatStar)*h2;
-  double m2 = b2*s2*s2;
-
-
-  double a = 0;
-
-  a += betatStar*c;
-  a -= beta[t]*c;
-  a -= 0.5*d*betatStar*betatStar;
-  a += 0.5*d*beta[t]*beta[t];
-  a += h2;
-  a -= h;
-  a += log(s);
-  a -= log(s2);
-  a += 0.5*((betatStar-m)/s)*((betatStar-m)/s);
-  a -= 0.5*((beta[t]-m2)/s2)*((beta[t]-m2)/s2);
-
-  if(exp(a)>gsl_rng_uniform()){
-    beta[t] = betatStar;
-    acc_beta += 1;
-  }
-
-  return;
-}
-
-
-
-
-
 
 void update_lambda_br(DoubleMatrix& lambda, DoubleMatrix& lambda_br,DoubleVector& xi_lambda, IntMatrix& breakpoints, IntMatrix& breakpointsStar, IntVector& K, IntVector& KStar, IntVector& Km1, double alpha_lambda, double beta_lambda, const LongMatrix& Y, const LongMatrix& Z, int n, int I, double& acceptedbr, const DoubleMatrix& omega, int theta_pred_estim, int xi_estim, int K_geom, double p_K, double alpha_xi, double beta_xi)
 {
@@ -1914,12 +1378,12 @@ LongMatrix surveillancedata2twin(int* x, int n, int I) {
   LongMatrix Z(I+1, n+1);
 
   /* Fill with zeros at all 0 index (rows & columns) */
-  for (register int t=0; t<=n; t++){ Z[0][t]=0; }
-  for (register int i=0; i<=I; i++){ Z[i][0]=0; }
+  for (int t=0; t<=n; t++){ Z[0][t]=0; }
+  for (int i=0; i<=I; i++){ Z[i][0]=0; }
 
   //Start @ index 1. (Z[0] is not defined)
-  for (register int t=1; t<=n; t++) {
-    for (register int i=1; i<=I; i++) { 
+  for (int t=1; t<=n; t++) {
+    for (int i=1; i<=I; i++) { 
       Z[i][t]=x[t-1]; 
     } 
   }
@@ -1929,112 +1393,6 @@ LongMatrix surveillancedata2twin(int* x, int n, int I) {
 
 
 
-/*********************************************************************
- * Read integer data from file. The format is: n Z_1 Z_2 ... Z_n
- *********************************************************************/
-
-// this could also be changed to return LongMatrix, but it is not used...
-// long** readData(char* fileName, long *size, long *size2) {
-//   //Read Data from file and check if everything went allright
-
-//   //cout << "\"" << fileName  << "\"" <<  endl;
-
-//   ifstream fin(fileName);
-  
-//   if (! fin) {
-//     cerr << "Error: File \"" << fileName << "\" not found!" << endl;
-//     return(NULL);
-//   }
-  
-//   // cout << fin << endl;
-//   //Count the number of data entries
-//   int n=0;
-//   fin >> n;
-//   Rprintf("n=%d\n",n);
-//   int I=1; 
-//   //fin >> I;
-//   //cout << "I=" << I << endl;
-
-//   long **Z = new long*[I+1];
-//   for (register long i=0; i<=I; i++){
-//     Z[i] = new long[n+1];
-//   }
-
-	
-//   for (register long t=0; t<=n; t++){
-//     Z[0][t]=0;
-//   }
-	
-//   for (register long i=0; i<=I; i++){
-//     Z[i][0]=0;
-//   }
-	
-//   //Start @ index 1. (Z[0] is not defined)
-//   int t=1;
-//   while (!fin.eof() && (t<=n)) { 	
-//     int i=1;
-//     while (!fin.eof() && (i<=I)) { 
-//       fin >> Z[i][t];
-//       i++;
-//     }
-//     t++;
-//   }
-
-//   fin.close();
-
-//   //Return the result consisting of Z and n
-//   *size = n;
-//   *size2 = I;
-
-//   return(Z);
-// }
-
-
-/* Calculate the deviance of the data we use that the data, Z, is a
- *  sum of Poisson distributed variables, i.e. it is Poisson
- *  distributed.
- *
- *    Z_t = S_t + X_t + Y_t, i.e. 
- *    Z_t ~ Po(nu*p + nu*(1-p) + lambda*W_{t-1})
- *
- *    D = -2log p(Z|theta) + 2 log p(Z|\mu(theta)=Z)
- */
-double satdevalt(int n, int I, const LongMatrix& X, const LongMatrix& Y, const LongMatrix& Z, 
-		 const DoubleMatrix& omega, const DoubleMatrix& lambda, const DoubleMatrix& nu, double *xi, DoubleMatrix& eta, DoubleMatrix& eta2, DoubleMatrix& varr, double psi, int overdispersion) {
-  double res = 0;
-  //Loop over all data
-  for (register int i=1; i<=I; i++) {
-    for (register int t=2; t<=n; t++) {
-      //Use the equation derived for the saturated deviance in the paper
-      //calculate the mean and variance of Z[i][t]
-      eta[i][t] = (nu[i][t]*xi[i]+lambda[i][t]*Z[i][t-1]);
-      eta2[i][t] = eta[i][t];
-      if(overdispersion){
-        varr[i][t] = eta2[i][t]*(1+eta2[i][t]/psi);
-      }else{
-	varr[i][t] = eta2[i][t];
-      }
-      //calculate the Deviance in the Poisson and NegBin case
-      if(!overdispersion){
-        if (Z[i][t] == 0) {
-          res += 2 * eta[i][t];
-        } else {
-          res += 2 * ( Z[i][t] * log(Z[i][t]/eta[i][t]) - Z[i][t] + eta[i][t]);
-        }
-      }
-      if(overdispersion){
-        if (Z[i][t] == 0) {
-          res += 2 * ( - (Z[i][t]+psi) * log((Z[i][t]+psi)/(eta[i][t]+psi)));
-        } else {
-          res += 2 * ( - (Z[i][t]+psi) * log((Z[i][t]+psi)/(eta[i][t]+psi)) + Z[i][t] * log(Z[i][t]/eta[i][t]));
-        }
-      }
-    }
-  }
-  return(res);
-}
-
-
 /* Calculate the deviance of the data we use that the data, Z, is a
  *  sum of Poisson distributed variables, i.e. it is Poisson
  *  distributed.
@@ -2048,8 +1406,8 @@ double satdev(int n, int I, const LongMatrix& Z,
 	      const DoubleMatrix& lambda, const DoubleMatrix& nu, double *xi, DoubleVector& epsilon, DoubleMatrix& eta, double psi, int overdispersion) {
   double res = 0;
   //Loop over all data
-  for (register int i=1; i<=I; i++) {
-    for (register int t=2; t<=n; t++) {
+  for (int i=1; i<=I; i++) {
+    for (int t=2; t<=n; t++) {
       //Use the equation derived for the saturated deviance in the paper
       //calculate the mean and variance of Z[i][t]
       eta[i][t] = (epsilon[t] + nu[i][t]*xi[i]+lambda[i][t]*Z[i][t-1]);
@@ -2067,37 +1425,6 @@ double satdev(int n, int I, const LongMatrix& Z,
 
 
 
-// Calculate chi square the sum of the qudratic pearson residuals (z-mean)/sd
-
-
-double chisq(int n, int I, const LongMatrix& Z, 
-	     const DoubleMatrix& lambda, const DoubleMatrix& nu, double *xi, DoubleVector& epsilon, DoubleMatrix& eta, DoubleMatrix& varr, DoubleMatrix& rpearson, double psi, int overdispersion) {
-  double res = 0;
-  //Loop over all data
-  for (register int i=1; i<=I; i++) {
-    for (register int t=2; t<=n; t++) {
-      //calculate the mean and variance of Z[i][t]
-      eta[i][t] = (epsilon[t] + nu[i][t]*xi[i]+lambda[i][t]*Z[i][t-1]);
-      if(overdispersion){
-        varr[i][t] = eta[i][t]*(1+eta[i][t]/psi);
-      }else{
-	varr[i][t] = eta[i][t];
-      }
-      rpearson[i][t] = (Z[i][t]-eta[i][t])/sqrt(varr[i][t]);
-      //calculate chisq in the Poisson and NegBin case
-      res += rpearson[i][t]*rpearson[i][t];
-    }
-  }
-  return(res);
-}
-  
-
-
-
-
-
-
-
 /**********************************************************************
  * Estimation in the basic epidemic model
  *
@@ -2178,7 +1505,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
   // We would have to delete the pointers manually at the end of the routine
   // in order not to corrupt the memory!!!
 
-  // for (register long i=0; i<=I; i++){
+  // for (long i=0; i<=I; i++){
   //   X[i]=new long[n+1];
   //   Y[i]=new long[n+1];
   //   S[i]=new long[n+1];
@@ -2240,21 +1567,21 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
   
   
   if(!varnu){
-    for (register int i=0; i<=I; i++) {
-      for (register int t=0; t<=n; t++) {
+    for (int i=0; i<=I; i++) {
+      for (int t=0; t<=n; t++) {
         nu[i][t] = alpha_nu/beta_nu;
       }
     }
   }
   
-  for (register int i=0; i<=I; i++) {
-    for (register int t=0; t<=n; t++) {
+  for (int i=0; i<=I; i++) {
+    for (int t=0; t<=n; t++) {
       lambda[i][t] = lambda_const;
     }
   }
   
-  for (register int i=0; i<=I; i++) {
-    for (register int t=0; t<=n; t++) {
+  for (int i=0; i<=I; i++) {
+    for (int t=0; t<=n; t++) {
       X[i][t] = 0; S[i][t] = 0; Y[i][t] = Z[i][t]; omega[i][t] = 1; eta[i][t] = 0; bp[i][t] = 0; bp_delta[t] = 0; bp_epsilon[t] = 0;
       sumX[i][t] = 0;  sumY[i][t] = 0;  sumS[i][t] = 0; sumomega[i][t] = 0; Sumeta[i][t] = 0; Sumrpearson[i][t] = 0;
     }
@@ -2412,14 +1739,14 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
   
   if(varnu){ 
-    for (register int i=0; i<=I; i++) {
+    for (int i=1; i<=I; i++) {
       alpha[i] = log(xreg[i]);
     }
     
-    for (register int t=0; t<=n; t++) {
+    for (int t=0; t<=n; t++) {
       beta[t] = 0.0;
     }
-    for (register int j=0; j<ncov; j++) {
+    for (int j=0; j<ncov; j++) {
       gamma[j] = 0.0;
     }
     
@@ -2441,7 +1768,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
   }//if varnu  
   
   
-  for (register int i=1;i<=I; i++) {
+  for (int i=1;i<=I; i++) {
     X[i][2] = (long)floor(nu[i][2]);
     Y[i][2] = (long)floor(lambda[i][2]*nu[i][2]/(1 - lambda[i][2]));
     omeganp1[i] = 1;
@@ -2460,8 +1787,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
   long acc_gamma=0;
 
 
-  /*hoehle: min/max is deprecated double tuneSampleSize = 1000<?burnin; */
-  double tuneSampleSize = MIN(1000,burnin);
+  double tuneSampleSize = (burnin < 1000 ? burnin : 1000);
 
   double need = 0; 
   double tunex = 0; 
@@ -2483,12 +1809,12 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
   }
 
   if(varnu){
-    for (register int j=0;j<ncov;j++) {
+    for (int j=0;j<ncov;j++) {
       logfile << "gamma[" << j << "]\t";
     }
     if(delta_rev){
       logfile << "Kdelta\t" << "xidelta\t";
-      for (register int j=2; j<=n; j++) {
+      for (int j=2; j<=n; j++) {
         logfile << "delta[" << j << "]\t";
       }
     }
@@ -2496,7 +1822,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
   if(epsilon_rev){
     logfile << "Kepsilon\t" << "xiepsilon\t";
-    for (register int j=2; j<=n; j++) {
+    for (int j=2; j<=n; j++) {
       logfile << "epsilon[" << j << "]\t";
     }
   }
@@ -2516,7 +1842,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
 
   //Write the header to the logfile2
-  for (register int t=1;t<=n;t++) {
+  for (int t=1;t<=n;t++) {
     logfile2 << "X[" << t << "]\tY[" << t << "]\tomega["
 	     << t << "]\tbp[" << t <<"]\t";
   }
@@ -2537,7 +1863,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
 
   /*Loop over samples - start at 1*/
-  register int sampleCounter=1;
+  int sampleCounter=1;
   while ( sampleCounter<=sampleSize) {
  
     if ((!verbose) && ((sampleCounter % 10) == 0)) {
@@ -2587,13 +1913,13 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
     double XSum = sumIn2(X,I,n);
  
     double xiSum = 0;
-    for (register int i=1;i<=I; i++) {
+    for (int i=1;i<=I; i++) {
       xiSum = xiSum + xi[i];
     }
 
     if(!la_estim){
-      for (register int i=1;i<=I; i++) {
-	for (register int t=2;t<=n; t++) { 
+      for (int i=1;i<=I; i++) {
+	for (int t=2;t<=n; t++) { 
 	  lambda[i][t] =  0;
 	}
       }
@@ -2614,8 +1940,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 	  logFx = log(gsl_ran_beta_pdf(lambda_const,alpha_lambda,beta_lambda)) + log(invlogitd(x));
 	  double logFxStar = 0;
 	  logFxStar = log(gsl_ran_beta_pdf(lambdaStar,alpha_lambda,beta_lambda)) + log(invlogitd(xStar));
-	  for (register int i=1;i<=I; i++) {
-	    for (register int t=2;t<=n; t++) {
+	  for (int i=1;i<=I; i++) {
+	    for (int t=2;t<=n; t++) {
 	      logFx = logFx + gsl_ran_poisson_log_pdf(Y[i][t], lambda_const*omega[i][t]*Z[i][t-1]);
 	      logFxStar = logFxStar + gsl_ran_poisson_log_pdf(Y[i][t], lambdaStar*omega[i][t]*Z[i][t-1]);
 	    }
@@ -2623,8 +1949,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 	  double accx = exp(logFxStar - logFx);
    
 	  if (gsl_rng_uniform() <= accx) {x = xStar; lambda_const = invlogit(xStar); acceptedlambda++;}
-	  for (register int i=1;i<=I; i++) {
-	    for (register int t=2;t<=n; t++) { 
+	  for (int i=1;i<=I; i++) {
+	    for (int t=2;t<=n; t++) { 
 	      lambda[i][t] =  lambda_const;
 	    }
 	  }
@@ -2634,8 +1960,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
     if(!varnu){
 
       double omegaxiSum = 0;
-      for (register int i=1;i<=I; i++) {
-	for (register int t=2;t<=n; t++) { 
+      for (int i=1;i<=I; i++) {
+	for (int t=2;t<=n; t++) { 
 	  omegaxiSum += omega[i][t]*xi[i];
 	}
       }
@@ -2644,8 +1970,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
       a  = alpha_nu + XSum;
       b  = beta_nu + omegaxiSum;
       nu_const =  gsl_ran_gamma (a, 1/b);
-      for (register int i=1;i<=I; i++) {
-	for (register int t=2;t<=n; t++) { 
+      for (int i=1;i<=I; i++) {
+	for (int t=2;t<=n; t++) { 
 	  nu[i][t] =  nu_const;
 	}
       }
@@ -2668,8 +1994,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
       }
       //loglik * prior of the old value
       double logFPsi = gsl_ran_gamma_log_pdf(psi,a_psi,1/b_psi) + log(psi);
-      for (register int i=1; i<=I; i++) {
-	for (register int t=2; t<=n; t++){
+      for (int i=1; i<=I; i++) {
+	for (int t=2; t<=n; t++){
 	  logFPsi += gsl_ran_gamma_log_pdf(omega[i][t],psi,1/psi);
 	}
       }
@@ -2678,8 +2004,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
       double psiStar = exp(logpsiStar);
       //loglik * prior of the new value
       double logFPsiStar = gsl_ran_gamma_log_pdf(psiStar,a_psi,1/b_psi) + logpsiStar;
-      for (register int i=1; i<=I; i++) {
-	for (register int t=2; t<=n; t++) {
+      for (int i=1; i<=I; i++) {
+	for (int t=2; t<=n; t++) {
 	  logFPsiStar += gsl_ran_gamma_log_pdf(omega[i][t],psiStar,1/psiStar);
 	}
       }
@@ -2748,22 +2074,6 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
       if (rw>0)
 	{
 
-	  //  update_beta_nurrw(gamma, alpha, beta, delta, X, Z, Y, n, I,  taubeta,  rw, 1, lambda, acc_beta, sampleCounter, my, my2, temp, z, theta, Q, Q2, L, L2, xcov, ncov, scov, omega, omega, 1);
-	  //update_beta_block(alpha, beta, gamma, delta, X, n, I, taubeta, rw, acc_beta, sampleCounter, n1, n2, my, my2, z, theta, beta0, Q, Q2, L, L2, xcov, ncov, scov, omega);
-
-	  /*hofmann - no fortran
-	    update_beta_tau_block(alpha, beta, gamma, delta, beta_a, beta_b, X, n, I, taubeta, rw, acc_beta, taubetaRWSigma, taubetaStar, sampleCounter, n1, n2, my, my2, z, theta, beta0, Q, Q2, L, L2, xcov, ncov, scov, omega);
-	  */
-	  //taubeta=beta_a/beta_b;
-	  // taubeta=hyper(rw, beta, beta_a, beta_b, n);
-	  //taubeta=720;
-
-	  //if(sampleCounter%500==1){cout << taubeta << endl << endl;}
-	  //  for(int t=2;t<=n;t++){
-	  //  update_beta_t(t, alpha, beta, gamma, delta, ncov, xcov, X, n, I, taubeta, acc_beta, omega, scov);
-	  //   }
-  
-	 
 	  if(scov==0){
 	    //  if (sampleCounter%1==0)
 	    //   {
@@ -2812,26 +2122,26 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
 
       if ((sampleCounter>burnin) && ((sampleCounter-burnin) % filter == 0)) {
-	//     for (register int i=1;i<=I; i++) {
-	//       for (register int t=1; t<=n; t++) {
+	//     for (int i=1;i<=I; i++) {
+	//       for (int t=1; t<=n; t++) {
 	// 	 logfile << nu[i][t] << "\t";
 	//       }
 	//     }
 	//    logfile << mu << "\t";
-	for (register int j=0; j<ncov; j++) {
+	for (int j=0; j<ncov; j++) {
 	  logfile << gamma[j] << "\t";
 	}
 	if(delta_rev){
 	  if ((sampleCounter>burnin) && ((sampleCounter-burnin) % filter == 0)) {
 	    logfile << Km1_delta<<"\t"<< xi_delta<<"\t";
-	    for (register int j=2; j<=n; j++) {
+	    for (int j=2; j<=n; j++) {
 	      logfile << delta[j] << "\t";
 	    }
 	  }
 
 	  if (sampleCounter>burnin) {
-	    for (register int k=1; k<=K_delta; k++) {
-	      for (register int j=2; j<=n; j++) {
+	    for (int k=1; k<=K_delta; k++) {
+	      for (int j=2; j<=n; j++) {
 		if (breakpoints_delta[k]==j){
 		  bp_delta[j]+=1;
 		}
@@ -2849,14 +2159,14 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
       if ((sampleCounter>burnin) && ((sampleCounter-burnin) % filter == 0)) {
         logfile << Km1_epsilon<<"\t"<< xi_epsilon<<"\t";
-	for (register int j=2; j<=n; j++) {
+	for (int j=2; j<=n; j++) {
 	  logfile << epsilon[j] << "\t";
         }
       }
 
       if (sampleCounter>burnin) {
-        for (register int k=1; k<=K_epsilon; k++) {
-          for (register int j=2; j<=n; j++) {
+        for (int k=1; k<=K_epsilon; k++) {
+          for (int j=2; j<=n; j++) {
             if (breakpoints_epsilon[k]==j){
   	      bp_epsilon[j]+=1;
             }
@@ -2875,14 +2185,14 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
 	  if ((sampleCounter>burnin) && ((sampleCounter-burnin) % filter == 0)) {
 	    logfile << Km1[1]<<"\t"<< xi_lambda[1]<<"\t";
-	    for (register int j=2; j<=n; j++) {
+	    for (int j=2; j<=n; j++) {
 	      logfile << lambda[1][j] << "\t";
 	    }
 	  }
-	  for (register int i=1;i<=I; i++) {
+	  for (int i=1;i<=I; i++) {
 	    if (sampleCounter>burnin) {
-	      for (register int k=1; k<=K[i]; k++) {
-		for (register int j=2; j<=n; j++) {
+	      for (int k=1; k<=K[i]; k++) {
+		for (int j=2; j<=n; j++) {
 		  if (breakpoints[i][k]==j){
 		    bp[i][j]+=1;
 		  }
@@ -2895,8 +2205,8 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
     // cout << S[1][106] << endl;
     //   cout << "test" << endl;
     //Loop over the individual X[t], Y[t], S[t], and omega[t]
-    for (register int i=1;i<=I; i++) {
-      for (register int t=2; t<=n; t++) {
+    for (int i=1;i<=I; i++) {
+      for (int t=2; t<=n; t++) {
         //Update X
         double binp = nu[i][t]*xi[i] / (epsilon[t] + nu[i][t]*xi[i] + lambda[i][t] * Z[i][t-1]);
 	X[i][t] =  gsl_ran_binomial( binp, Z[i][t]);
@@ -2934,7 +2244,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
 
     //Praediktive Verteilung fuer variables nu
-    for (register int i=1;i<=I;i++) {
+    for (int i=1;i<=I;i++) {
       if(!theta_pred_estim){
 	double p_thetanp1 = ((double(K[i]))/double(n)); //(1+double(K[i]))
 	if(K_geom){
@@ -3069,7 +2379,7 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
 
   //Write means to logfile2
 
-  for (register int t=1;t<=n;t++) {
+  for (int t=1;t<=n;t++) {
     logfile2 << (double)sumX[1][t]/((double)samples*(double)filter) << "\t" << (double)sumY[1][t]/((double)samples*(double)filter)<< "\t" << (double)sumomega[1][t]/((double)samples*(double)filter) << "\t"<< (double)bp[1][t]/((double)samples*(double)filter) << "\t";
   }
   logfile2 << (double)bp[1][n+1]/((double)samples*(double)filter) << "\t";
@@ -3089,9 +2399,9 @@ void bplem_estimate(int verbose, ofstream &logfile, ofstream &logfile2, ofstream
   return;
     
 
+}//bplem_estimate
 
 
-}//funktion
 
 /* hoehle: interface for calling twins from R */
 
@@ -3181,7 +2491,7 @@ extern "C" {
   ofstream logfile,logfile2,accfile;
 
   char accFile[200];
-  sprintf(accFile, "%s.acc", logFile);
+  snprintf(accFile, 200, "%s.acc", logFile);
 
   logfile.open(logFile);
   logfile2.open(logFile2);
